import logging
from typing import Optional, Dict
from requests import HTTPError
from .base import ResourceHandler

logger = logging.getLogger(__name__)


class ComputerConfigProfileHandler(ResourceHandler):
    resource_name = "macOS Configuration Profile"

    def delete(self, resource_id: int) -> bool:
        return self.client.classic.configuration_profiles.delete_by_id(resource_id)

    def get(self, resource_id: int) -> Optional[Dict]:
        try:
            return self.client.classic.configuration_profiles.get_by_id(
                resource_id
            ).json()
        except HTTPError as e:
            logger.error(
                "Could not retrieve %s %s: %s", self.resource_name, resource_id, e
            )
            return None



from typing import Dict, Type
from .handlers import (
    ResourceHandler,
    ComputerAttributeHandler,
    ComputerConfigProfileHandler,
    ComputerGroupHandler,
    ComputerHandler,
    ScriptHandler,
    PackageHandler,
    PolicyHandler,
    RestrictedSoftwareHandler,
    MacAppsHandler,
)


class ResourceRegistry:
    """Central Registry for resource type mappings"""

    def __init__(self):
        self._handlers: Dict[str, Type[ResourceHandler]] = {}
        self._register_default_handlers()

    def _register_default_handlers(self):
        """Register all default resource handlers"""
        self.register("unusedComputers", ComputerHandler)
        self.register("unusedComputerGroups", ComputerGroupHandler)
        self.register("unusedMacApps", MacAppsHandler)
        self.register("unusedMobileDeviceApps", MacAppsHandler)
        self.register("unusedPackages", PackageHandler)
        self.register("unusedPolicies", PolicyHandler)
        self.register("unusedComputerProfiles", ComputerConfigProfileHandler)
        self.register("unusedScripts", ScriptHandler)
        self.register("unusedComputerEAs", ComputerAttributeHandler)
        self.register("unusedRestrictedSoftware", RestrictedSoftwareHandler)

    def register(self, resource_type: str, handler_class: Type[ResourceHandler]):
        """Register a handler for a resource type"""
        self._handlers[resource_type] = handler_class

    def get_handler_class(self, resource_type: str) -> Type[ResourceHandler]:
        """Get the handler class for a resource type"""
        return self._handlers.get(resource_type)

    def is_registered(self, resource_type: str) -> bool:
        """Check if a resource type is registered"""
        return resource_type in self._handlers



from dataclasses import dataclass
from typing import Optional, Dict
from enum import Enum


class OperationStatus(Enum):
    SUCCESS = "success"
    FAILED = "failed"
    SKIPPED = "skipped"


@dataclass
class DeletionResult:
    """Result of a deletion operation"""

    resource_type: str
    resource_id: int
    resource_name: str
    status: OperationStatus
    backup_data: Optional[Dict] = None
    error_message: Optional[str] = None

    @property
    def success(self) -> bool:
        return self.status == OperationStatus.SUCCESS


@dataclass
class BatchResult:
    """Result of a batch deletion operation"""

    total_processed: int
    successful: int
    failed: int
    skipped: int
    results: list[DeletionResult]
    backup_path: Optional[str] = None



import logging
from typing import Optional, Dict
from requests import HTTPError
from .base import ResourceHandler

logger = logging.getLogger(__name__)

# TODO This needs tested to see what the API returns? Does it return the package or does it return metadata


class PackageHandler(ResourceHandler):
    resource_name = "Package"

    def delete(self, resource_id: int) -> bool:
        return self.client.classic.packages.delete_by_id(resource_id)

    def get(self, resource_id: int) -> Optional[Dict]:
        try:
            return self.client.classic.packages.get_by_id(resource_id)
        except HTTPError as e:
            logger.error(
                "Could not retrieve %s %s: %s", self.resource_name, resource_id, e
            )
            return None



from abc import ABC, abstractmethod
from jamfpy import Tenant


class ResourceHandler(ABC):
    """Base Class for resource handling"""

    def __init__(self, client: Tenant):
        self.client = client

    @abstractmethod
    def delete(self, resource_id: int) -> bool:
        """Delete a resource by ID"""
        pass

    @abstractmethod
    def get(self, resource_id: int) -> bool:
        """Get a resource by ID"""
        pass

    @property
    @abstractmethod
    def resource_name(self) -> str:
        """Human readable resource name"""
        pass



from pathlib import Path
from datetime import datetime
from typing import Dict, List
import json


class BackupManager:
    """Handles backup operations of Jamf Resources"""

    def __init__(self, backup_dir: Path):
        self.backup_dir = backup_dir
        self.backup_dir.mkdir(parents=True, exist_ok=True)

    def save_backup(self, backup_data: Dict, timestamp: str = None) -> str:
        """Save backup data to a JSON file"""

        if timestamp is None:
            timestamp = datetime.now().strftime("%d%m%Y_%H%M%S")

        backup_filename = f"backup_{timestamp}.json"
        backup_path = self.backup_dir / backup_filename

        with open(backup_path, "w") as f:
            json.dump(backup_data, f, indent=2)

        return str(backup_path)

    def list_backups(self) -> List[str]:
        """List all backup files in chronological order"""
        backup_files = sorted(self.backup_dir.glob("backup_*.json"), reverse=True)
        return [f.name for f in backup_files]

    def load_backup(self, backup_filename: str) -> Dict:
        """Load a backup file"""

        backup_path = self.backup_dir / backup_filename

        if not backup_path.exists():
            raise FileNotFoundError(f"Backup file not found: {backup_path}")

        with open(backup_path, "r") as f:
            return json.load(f)



from .base import ResourceHandler

# TODO - Need to test this and make sure that I can
# delete all app types and that I can not export them.


class MacAppsHandler(ResourceHandler):
    resource_name = "Mac App"

    def delete(self, resource_id: int) -> bool:
        return self.client.pro.app_installers.delete(resource_id)



import logging
from typing import Optional, Dict
from requests import HTTPError
from .base import ResourceHandler

logger = logging.getLogger(__name__)


class ComputerAttributeHandler(ResourceHandler):
    resource_name = "Computer Extension Attribute"

    def delete(self, resource_id: int) -> bool:
        return self.client.classic.computer_extension_attributes.delete_by_id(
            resource_id
        )

    def get(self, resource_id: int) -> Optional[Dict]:
        try:
            return self.client.classic.computer_extension_attributes.get_by_id(
                resource_id
            ).json()
        except HTTPError as e:
            logger.error(
                "Could not retrieve %s %s: %s", self.resource_name, resource_id, e
            )
            return None



def check():
    assert 1 + 1 == 2


# import pytest
# from unittest.mock import Mock

# from jamf_resource_deleter import JamfResourceDeleter
# from jamf_resource_deleter.models import OperationStatus


# @pytest.fixture
# def mock_client():
#     return Mock()


# @pytest.fixture
# def deleter(mock_client, tmp_path):
#     return JamfResourceDeleter(mock_client, backup_dir=str(tmp_path))


# def test_delete_resource_success(deleter, mock_client):
#     # Setup mock
#     mock_client.classic.computers.delete_by_id.return_value = True

#     # Execute
#     result = deleter.delete_resource(
#         "unusedComputers", resource_id=123, resource_name="Test Computer"
#     )

#     # Assert
#     assert result.status == OperationStatus.SUCCESS
#     assert result.resource_id == 123
#     mock_client.classic.computers.delete_by_id.assert_called_once_with(123)



from .base import ResourceHandler
from .computer_attribute_handler import ComputerAttributeHandler
from .computer_handler import ComputerHandler
from .comupter_group_handler import ComputerGroupHandler
from .config_handler import ComputerConfigProfileHandler
from .mac_app_handler import MacAppsHandler
from .package_handler import PackageHandler
from .policy_handler import PolicyHandler
from .restricted_software_handler import RestrictedSoftwareHandler
from .script_handler import ScriptHandler

__all__ = [
    "ResourceHandler",
    "ComputerAttributeHandler",
    "ComputerHandler",
    "ComputerGroupHandler",
    "ComputerConfigProfileHandler",
    "MacAppsHandler",
    "PackageHandler",
    "PolicyHandler",
    "RestrictedSoftwareHandler",
    "ScriptHandler",
]



"""This module takes in some JSON files and loops through to gather IDs
of resources from Jamf Pro and then loops through and deletes the resources.
"""

from typing import Optional
from pathlib import Path
from datetime import datetime
import json
import logging

from jamfpy import Tenant
from .backup_manager import BackupManager
from .resource_registry import ResourceRegistry
from .models import DeletionResult, BatchResult, OperationStatus

logger = logging.getLogger(__name__)


class JamfResourceDeleter:
    """
    Class that will workout which resources need deleted and then delete them
    """

    def __init__(
        self,
        jamfpy_client: Tenant,
        backup_dir: Optional[str] = None,
        registry: Optional[ResourceRegistry] = None,
        configure_logging: bool = True
    ):
        """

        Args:
            jamfpy_client (Tenant): _description_
            backup_dir (Optional[str], optional): _description_. Defaults to None.
            registry (Optional[ResourceRegistry], optional): _description_. Defaults to None.
        """

        if configure_logging:
            root_logger = logging.getLogger()
            if not root_logger.handlers:
                logging.basicConfig(
                    level=logging.INFO,
                    format='%(levelname)s - %(message)s'
                )
        
        self.logger = logging.getLogger(__name__)

        self.client = jamfpy_client
        self.backup_dir = backup_dir

        if backup_dir:
            backup_path = Path(backup_dir)
        else:
            backup_path = Path(__file__).parent / "backups"

        self.backup_manager = BackupManager(backup_path)

        self.registry = registry or ResourceRegistry()

    def delete_resource(
        self,
        resource_type: str,
        resource_id: int,
        resource_name: str = "Unknown",
        export: bool = False,
    ) -> DeletionResult:
        """Delete a single resource

        Args:
            resource_type (str): _description_
            resource_id (int): _description_
            resource_name (str, optional): _description_. Defaults to "Unknown".
            export (bool, optional): _description_. Defaults to False.

        Returns:
            DeletionResult: _description_
        """

        handler_class = self.registry.get_handler_class(resource_type)

        if not handler_class:
            logger.error("Unknown resource type: %s", resource_type)
            return DeletionResult(
                resource_type=resource_type,
                resource_id=resource_id,
                resource_name=resource_name,
                status=OperationStatus.FAILED,
                error_message=f"Unkown resource type: {resource_type}",
            )

        handler = handler_class(self.client)
        backup_data = None

        if export:
            backup_data = handler.get(resource_id)
            if backup_data:
                logger.info(
                    "Backed up %s %s (ID: %s)",
                    resource_type,
                    resource_name,
                    resource_id,
                )

        try:
            success = handler.delete(resource_id)

            if success:
                logger.info(
                    "Successfully deleted %s %s (ID: %s)",
                    resource_type,
                    resource_name,
                    resource_id,
                )
                return DeletionResult(
                    resource_type=resource_type,
                    resource_id=resource_id,
                    resource_name=resource_name,
                    status=OperationStatus.SUCCESS,
                    backup_data=backup_data,
                )
            else:
                logger.warning(
                    "Failed to delete %s %s (ID: %s)",
                    resource_type,
                    resource_name,
                    resource_id,
                )
                return DeletionResult(
                    resource_type=resource_type,
                    resource_id=resource_id,
                    resource_name=resource_name,
                    status=OperationStatus.FAILED,
                    error_message="Deletion returned false",
                )
        except Exception as e:
            logger.error(
                "Error deleting %s %s (ID: %s)",
                resource_type,
                resource_name,
                resource_id,
            )
            return DeletionResult(
                resource_type=resource_type,
                resource_id=resource_id,
                resource_name=resource_name,
                status=OperationStatus.FAILED,
                error_message=str(e),
                backup_data=backup_data,
            )

    def delete_from_json(
        self, json_file_path: Path, dry_run: bool = True, export: bool = False
    ) -> BatchResult:
        """Delete resources from a JSON file

        Args:
            json_file_path (Path): _description_
            dry_run (bool, optional): _description_. Defaults to True.
            export (bool, optional): _description_. Defaults to False.

        Returns:
            BatchResult: _description_
        """

        if not json_file_path.exists():
            raise FileNotFoundError(f"JSON file not found: {json_file_path}")

        with open(json_file_path, "r") as f:
            unused_resources = json.load(f)

        timestamp = datetime.now().strftime("%d%m%Y%H%M%S")
        session_backups: dict[str, dict] = {}
        results = []

        for resource_type, resource_list in unused_resources.items():
            logger.info("Processing %s...", resource_type)

            if export and not dry_run:
                session_backups[resource_type] = []

            for resource in resource_list:
                resource_id = resource.get("id")
                resource_name = resource.get("name")

                if dry_run:
                    logger.info(
                        "[DRY-RUN] Would delete %s:%s (ID: %s)",
                        resource_type,
                        resource_name,
                        resource_id,
                    )
                    if export:
                        logger.info("[DRY-RUN] Would backup configuration first")

                    results.append(
                        DeletionResult(
                            resource_type=resource_type,
                            resource_id=resource_id,
                            resource_name=resource_name,
                            status=OperationStatus.SKIPPED,
                        )
                    )
                else:
                    result = self.delete_resource(
                        resource_type=resource_type,
                        resource_id=resource_id,
                        resource_name=resource_name,
                        export=export,
                    )
                    results.append(result)

                    if export and result.backup_data:
                        session_backups[resource_type].append(
                            {
                                "id": resource_id,
                                "name": resource_name,
                                "configuration": result.backup_data,
                            }
                        )

        backup_path = None
        if export and not dry_run and session_backups:
            backup_path = self.backup_manager.save_backup(session_backups, timestamp)
            logger.info("Backup saved to: %s", backup_path)

        batch_result = BatchResult(
            total_processed=len(results),
            successful=sum(1 for r in results if r.status == OperationStatus.SUCCESS),
            failed=sum(1 for r in results if r.status == OperationStatus.FAILED),
            skipped=sum(1 for r in results if r.status == OperationStatus.SKIPPED),
            results=results,
            backup_path=backup_path,
        )

        return batch_result

    def list_backups(self) -> list:
        """List all backup files"""
        return self.backup_manager.list_backups()

    def restore_from_backup(self, backup_filename: str, dry_run: bool = True):
        """TO_DO WORK IN PROGRESS"""

        backup_path = self.backup_dir / backup_filename

        if not backup_path.exists():
            raise FileNotFoundError(f"Backup file not found: {backup_path}")

        with open(backup_path, "r") as f:
            backup_data = json.load(f)

        print(f"\n{'=' * 50}")
        print(f"Restoring from backup: {backup_filename}")
        print(f"{'=' * 50}")

        for resource_type, resources in backup_data.items():
            print(f"\nRestoring {resource_type}...")

            for resource in resources:
                resource_name = resource.get("name", "Unkown")
                # resource_config = resource.get("configuration")

            if dry_run:
                print(f"[DRY-RUN] Would restore {resource_type}: {resource_name}")
            else:
                print(f"Restoring {resource_type}: {resource_name}...")
                # TODO Add in logic here for restoring from file to Jamf Pro
                # Needs more thinking
                print("Restoration not implemented yet")
